package iso;
import java.awt.Color;
import figure.Cube3D;
import figure.Point3D;
import figure.Triangle3D;
import figure.TriangleCollection;
import iso.component.IsoSurfaceList;

//--------------------------------------------------------------------//
/**
 * `lf[^f<br>
 * f[^̔(y<=0 ̈)̃f[^
 */
public class ChannelFlowModel{

/** ǖʑ */
  public static final Attribute
    WALL_ATTR=new Attribute(-1,0.0,new Color(0,102,0));

/** <i>x</i> f[^ */
  public int nx;

/** <i>y</i> f[^ */
  public int ny;

/** <i>z</i> f[^ */
  public int nz;

/** <i>x</i> W */
  public double x[];

/** <i>y</i> W */
  public double y[];

/** <i>z</i> W */
  public double z[];

/**  <i>x</i>  u */
  public double u[][][];

/**  <i>y</i>  v */
  public double v[][][];

/**  <i>z</i>  w */
  public double w[][][];

/** ϓ */
  public double p[][][];

/**  uv (u>0) */
  public double uv1[][][];

/**  uv (u<0) */
  public double uv2[][][];

/** fE\ <code>Cube3D</code> */
  public Cube3D bounds;

/** lʃXg */
  protected IsoSurfaceList list;

/** ǖ */
  public IsoSurface wall;

/**  */
  public Point3D lightPoint;

/**
 * `lf[^f\z
 * @param list lʃXg
 */
  public ChannelFlowModel(IsoSurfaceList list){
    this.list=list;
    nx=0;
    ny=0;
    nz=0;
    bounds=new Cube3D();
    wall=null;
    lightPoint=new Point3D();
  }

//--------------------------------------------------------------------//
/**
 * f[^ݒ肷
 * @param nnx f[^ <i>x</i> 
 * @param nny f[^ <i>y</i> 
 * @param nnz f[^ <i>z</i> 
 * @param lx <i>x</i> ̈撷
 * @param lz <i>z</i> ̈撷
 * @param uu f[^̗ <i>x</i> 
 * @param vv f[^̗ <i>y</i> 
 * @param ww f[^̗ <i>z</i> 
 * @param pp f[^̕ϓ
 */
  public void set(int nnx,int nny,int nnz,double lx,double lz,
      	      	  double uu[][][],double vv[][][],double ww[][][],
      	      	  double pp[][][]){
    nx=nnx+1;
    ny=nny/2+1;
    nz=nnz+1;
    x=new double[nx];
    y=new double[ny];
    z=new double[nz];
    u=new double[nx][ny][nz];
    v=new double[nx][ny][nz];
    w=new double[nx][ny][nz];
    p=new double[nx][ny][nz];
    uv1=new double[nx][ny][nz];
    uv2=new double[nx][ny][nz];
    double dx=lx/(double)nnx;
    double dth=Math.PI/(double)nny;
    double dz=lz/(double)nnz;
    for(int i=0;i<nx;i++){
      x[i]=dx*(double)i;
    }
    for(int j=0;j<ny;j++){
      y[j]=Math.cos(dth*(double)(nny-j));
    }
    for(int k=0;k<nz;k++){
      z[k]=dz*(double)k;
    }
    bounds.setBounds(0.0,-1.0,0.0,lx,1.0,lz);
    double scoef=1.0/(double)(nnx*nnz);
    for(int j=0;j<ny;j++){
      int jj=nny-j;
      double su=0.0;
      double sv=0.0;
      double sw=0.0;
      double sp=0.0;
      for(int i=0;i<nnx;i++){
      	for(int k=0;k<nnz;k++){
      	  su+=uu[i][jj][k];
      	  sv+=vv[i][jj][k];
      	  sw+=ww[i][jj][k];
      	  sp+=pp[i][jj][k];
      	}
      }
      su*=scoef;
      sv*=scoef;
      sw*=scoef;
      sp*=scoef;
      for(int i=0;i<nx;i++){
      	int ii=i%nnx;
      	for(int k=0;k<nz;k++){
      	  int kk=k%nnz;
      	  u[i][j][k]=uu[ii][jj][kk]-su;
      	  v[i][j][k]=vv[ii][jj][kk]-sv;
      	  w[i][j][k]=ww[ii][jj][kk]-sw;
      	  p[i][j][k]=pp[ii][jj][kk]-sp;
      	  if(u[i][j][k]>=0.0){
      	    uv1[i][j][k]=u[i][j][k]*v[i][j][k];
      	    uv2[i][j][k]=0.0;
      	  }
      	  else{
      	    uv1[i][j][k]=0.0;
      	    uv2[i][j][k]=u[i][j][k]*v[i][j][k];
      	  }
      	}
      }
    }
    TriangleCollection wtc=new TriangleCollection();
    wtc.add(new Facet(new Point3D(0.0,-1.0,0.0),
      	      	      new Point3D(lx,-1.0,0.0),
      	      	      new Point3D(lx,-1.0,lz)));
    wtc.add(new Facet(new Point3D(0.0,-1.0,0.0),
      	      	      new Point3D(lx,-1.0,lz),
      	      	      new Point3D(0.0,-1.0,lz)));
    wall=new IsoSurface(wtc,WALL_ATTR);
    lightPoint.setLocation(0.5*lx,2.0,3.0*lz);
    resetSurface();
  }

//--------------------------------------------------------------------//
/**
 * lʂĐݒ肷
 */
  protected void resetSurface(){
    int count=list.getSurfaceCount();
    Attribute attr[]=new Attribute[count];
    for(int i=0;i<count;i++){
      attr[i]=list.getIsoSurface(i).attr;
    }
    list.clear();
    for(int i=0;i<count;i++){
      IsoSurface s=createIsoSurface(attr[i]);
      if(s!=null){
      	list.addElement(s);
      }
    }
  }

//--------------------------------------------------------------------//
/**
 * f[^폜
 */
  public void clear(){
    nx=0;
    ny=0;
    nz=0;
    x=new double[nx];
    y=new double[ny];
    z=new double[nz];
    u=new double[nx][ny][nz];
    v=new double[nx][ny][nz];
    w=new double[nx][ny][nz];
    p=new double[nx][ny][nz];
    uv1=new double[nx][ny][nz];
    uv2=new double[nx][ny][nz];
    list.clear();
    wall=null;
  }

//--------------------------------------------------------------------//
/**
 * lʂ쐬
 * @param attr lʂ̑
 * @return l
 */
  public IsoSurface createIsoSurface(Attribute attr){
    if((nx==0) || (ny==0) || (nz==0)) return null;
    IsoSurfaceData isd=null;
    switch(attr.type){
      case Attribute.U:
      	isd=new IsoSurfaceData(nx,ny,nz,x,y,z,u,attr.thres);
      	break;
      case Attribute.V:
      	isd=new IsoSurfaceData(nx,ny,nz,x,y,z,v,attr.thres);
      	break;
      case Attribute.W:
      	isd=new IsoSurfaceData(nx,ny,nz,x,y,z,w,attr.thres);
      	break;
      case Attribute.P:
      	isd=new IsoSurfaceData(nx,ny,nz,x,y,z,p,attr.thres);
      	break;
      case Attribute.UV1:
      	isd=new IsoSurfaceData(nx,ny,nz,x,y,z,uv1,attr.thres);
      	break;
      case Attribute.UV2:
      	isd=new IsoSurfaceData(nx,ny,nz,x,y,z,uv2,attr.thres);
      	break;
    }
    if(isd==null) return null;
    return new IsoSurface(isd.createFacet(),attr);
  }
}
