package iso;
import java.awt.Color;
import java.awt.Point;
import java.util.Arrays;
import figure.Point3D;
import figure.Transform;
import figure.Triangle3D;
import figure.TriangleCollection;
import iso.component.IsoSurfaceList;
import shading.FlatShading;
import shading.Shading;

//--------------------------------------------------------------------//
/**
 * `l\f
 */
public class ChannelFlowViewModel{

/** ̍ŏl */
  public static final double MIN_VALUE=1.0E-20;

/** wiF */
  public static final Color BG_COLOR=new Color(0,0,0);

/** `lf[^f */
  protected ChannelFlowModel model;

/** lʃXg */
  protected IsoSurfaceList list;

/** ϊs */
  protected Transform transform;

/** ʒ_ */
  protected Point centerPoint;

/** ʕ */
  protected int width;

/** ʍ */
  protected int height;

/** sNZf[^ */
  protected int pixel[];

/** Z obt@ */
  protected double zbuffer[];

/** VF[fBO */
  protected Shading shading;

/** f̕\{ */
  protected double vCoef;

/** f̒S */
  protected Point3D center;

/**  */
  protected Point3D lightPoint;

/**
 * `l\pl\z
 * @param model `lf[^f
 * @param list lʃXg
 * @param transform ϊs
 */
  public ChannelFlowViewModel(ChannelFlowModel model,
      	      	      	      IsoSurfaceList list,
      	      	      	      Transform transform){
    this.model=model;
    this.list=list;
    this.transform=transform;
    pixel=new int[0];
    zbuffer=new double[0];
    shading=null;
    vCoef=1.0;
    center=new Point3D();
    lightPoint=new Point3D();
  }

//--------------------------------------------------------------------//
/**
 * `lf[^fԂ
 * @return `lf[^f
 */
  public ChannelFlowModel getModel(){
    return model;
  }

//--------------------------------------------------------------------//
/**
 * ϊsԂ
 * @return ϊs
 */
  public Transform getTransform(){
    return transform;
  }

//--------------------------------------------------------------------//
/**
 * ʏݒ肷
 * @param centerPoint ʒ_
 * @param width ʕ
 * @param height ʍ
 */
  public void setView(Point centerPoint,int width,int height){
    this.centerPoint=centerPoint;
    this.width=width;
    this.height=height;
    pixel=new int[width*height];
    zbuffer=new double[width*height];
    initPixel();
    shading=new FlatShading(centerPoint,width,height,lightPoint);
  }

//--------------------------------------------------------------------//
/**
 * sNZf[^
 */
  public void initPixel(){
    Arrays.fill(pixel,BG_COLOR.getRGB());
    Arrays.fill(zbuffer,-Double.MAX_VALUE);
  }

//--------------------------------------------------------------------//
/**
 * lʂǉ
 * @param attr lʂ̑
 */
  public void addIsoSurface(Attribute attr){
    IsoSurface s=model.createIsoSurface(attr);
    if(s!=null){
      resetCoordinates(s);
      list.addElement(s);
    }
  }

//--------------------------------------------------------------------//
/**
 * lʂu
 * @param attr lʂ̑
 */
  public void setIsoSurface(Attribute attr){
    IsoSurface s=model.createIsoSurface(attr);
    if(s!=null){
      resetCoordinates(s);
      list.setIsoSurface(s);
    }
  }

//--------------------------------------------------------------------//
/**
 * WʍWɕϊ
 */
  public void resetCoordinates(){
    double hh=Math.max(model.bounds.height,model.bounds.depth);
    vCoef=0.8*Math.min((double)width/model.bounds.width,
      	      	       (double)height/hh);
    center=model.bounds.getCenterLocation();
    int count=list.getSurfaceCount();
    for(int i=0;i<count;i++){
      resetCoordinates(list.getIsoSurface(i));
    }
    if(model.wall!=null){
      resetCoordinates(model.wall);
    }
    resetCoordinates(model.lightPoint);
  }

//--------------------------------------------------------------------//
/**
 * WʍWɕϊ
 * @param s l
 */
  protected void resetCoordinates(IsoSurface s){
    int count=s.getCount();
    for(int i=0;i<count;i++){
      Facet facet=s.get(i);
      resetCoordinates(facet.v1);
      resetCoordinates(facet.v2);
      resetCoordinates(facet.v3);
      facet.setInit();
    }
  }

//--------------------------------------------------------------------//
/**
 * WʍWɕϊ
 * @param p _
 */
  protected void resetCoordinates(Point3D p){
    p.setLocation(vCoef*(p.x-center.x),vCoef*(p.y-center.y),
      	      	  vCoef*(p.z-center.z));
  }

//--------------------------------------------------------------------//
/**
 * VF[fBÕsNZf[^Ԃ
 * @return VF[fBÕsNZf[^
 */
  public int[] getPixel(){
    initPixel();
    int count=list.getSurfaceCount();
    lightPoint.setLocation(model.lightPoint);
    transform.trans(lightPoint);
    for(int i=0;i<count;i++){
      shading(list.getIsoSurface(i));
    }
    if(model.wall!=null){
      shading(model.wall);
    }
    return pixel;
  }

//--------------------------------------------------------------------//
/**
 * lʂ̃VF[fBO`vZ
 * @param s l
 */
  protected void shading(IsoSurface s){
    s.trans(transform);
    shading.shading(s.facets,s.attr.color,pixel,zbuffer);
  }
}
