package iso;
import figure.Point3D;
import figure.TriangleCollection;

//--------------------------------------------------------------------//
/**
 * lʃf[^
 */
public class IsoSurfaceData{

/** ۓ */
  public static final double TOL=1.0e-10;

/** <i>x</i> f[^ */
  protected int nx;

/** <i>y</i> f[^ */
  protected int ny;

/** <i>z</i> f[^ */
  protected int nz;

/** <i>x</i> W */
  protected double x[];

/** <i>y</i> W */
  protected double y[];

/** <i>z</i> W */
  protected double z[];

/** f[^z */
  protected double s[][][];

/** 臒l̏^\z */
  protected boolean ud[][][];

/** t@Zbg_ <i>x</i> W */
  protected double fx[][][];

/** t@Zbg_ <i>y</i> W */
  protected double fy[][][];

/** t@Zbg_ <i>z</i> W */
  protected double fz[][][];

/** 臒l */
  protected double thres;

/**
 * lʃf[^\z
 * @param nx <i>x</i> f[^
 * @param ny <i>y</i> f[^
 * @param nz <i>z</i> f[^
 * @param x <i>x</i> W
 * @param y <i>y</i> W
 * @param z <i>z</i> W
 * @param u f[^z
 * @param thres 臒l
 */
  public IsoSurfaceData(int nx,int ny,int nz,
      	      	      	double x[],double y[],double z[],
      	      	      	double u[][][],double thres){
    this.nx=nx;
    this.ny=ny;
    this.nz=nz;
    this.x=x;
    this.y=y;
    this.z=z;
    setArray(u,thres);
  }

//--------------------------------------------------------------------//
/**
 * f[^zݒ肷
 * @param u f[^z
 * @param thres 臒l
 */
  protected void setArray(double u[][][],double thres){
    this.thres=thres;
    s=new double[nx][ny][nz];
    ud=new boolean[nx][ny][nz];
    fx=new double[nx-1][ny][nz];
    fy=new double[nx][ny-1][nz];
    fz=new double[nx][ny][nz-1];
    for(int i=0;i<nx;i++){
      for(int j=0;j<ny;j++){
      	for(int k=0;k<nz;k++){
      	  s[i][j][k]=u[i][j][k]-thres;
      	  if(Math.abs(s[i][j][k])<TOL){
      	    if(s[i][j][k]>=0.0){
      	      s[i][j][k]=TOL;
      	    }
      	    else{
      	      s[i][j][k]=-TOL;
      	    }
      	  }
      	  ud[i][j][k]=(s[i][j][k]>0);
      	}
      }
    }
    for(int i=0;i<nx-1;i++){
      for(int j=0;j<ny;j++){
      	for(int k=0;k<nz;k++){
      	  if(ud[i][j][k]!=ud[i+1][j][k]){
      	    double cf=s[i][j][k]/(s[i][j][k]-s[i+1][j][k]);
      	    fx[i][j][k]=x[i]+cf*(x[i+1]-x[i]);
      	  }
      	  else{
      	    fx[i][j][k]=x[i];
      	  }
      	}
      }
    }
    for(int i=0;i<nx;i++){
      for(int j=0;j<ny-1;j++){
      	for(int k=0;k<nz;k++){
      	  if(ud[i][j][k]!=ud[i][j+1][k]){
      	    double cf=s[i][j][k]/(s[i][j][k]-s[i][j+1][k]);
      	    fy[i][j][k]=y[j]+cf*(y[j+1]-y[j]);
      	  }
      	  else{
      	    fy[i][j][k]=y[j];
      	  }
      	}
      }
    }
    for(int i=0;i<nx;i++){
      for(int j=0;j<ny;j++){
      	for(int k=0;k<nz-1;k++){
      	  if(ud[i][j][k]!=ud[i][j][k+1]){
      	    double cf=s[i][j][k]/(s[i][j][k]-s[i][j][k+1]);
      	    fz[i][j][k]=z[k]+cf*(z[k+1]-z[k]);
      	  }
      	  else{
      	    fz[i][j][k]=z[k];
      	  }
      	}
      }
    }
  }

//--------------------------------------------------------------------//
/**
 * lʂ쐬
 * @return l
 */
  public TriangleCollection createFacet(){
    TriangleCollection facets=new TriangleCollection();
    boolean v[]=new boolean[8];
    for(int i=0;i<nx-1;i++){
      for(int j=0;j<ny-1;j++){
      	for(int k=0;k<nz-1;k++){
      	  v[0]=ud[i][j][k];
      	  v[1]=ud[i+1][j][k];
      	  v[2]=ud[i+1][j+1][k];
      	  v[3]=ud[i][j+1][k];
      	  v[4]=ud[i][j][k+1];
      	  v[5]=ud[i+1][j][k+1];
      	  v[6]=ud[i+1][j+1][k+1];
      	  v[7]=ud[i][j+1][k+1];
      	  int index=MarchingCubes.cubeIndex(v);
      	  if((0<index) && (index<255)){
      	    int facet[]=MarchingCubes.FACET[index];
      	    int ii=0;
      	    while((ii<12) && (facet[ii]!=-1)){
      	      Point3D v1=facetVertex(facet[ii],i,j,k);
      	      Point3D v2=facetVertex(facet[ii+1],i,j,k);
      	      Point3D v3=facetVertex(facet[ii+2],i,j,k);
      	      if(thres<0.0){
      	      	facets.add(new Facet(v1,v2,v3));
      	      }
      	      else{
      	      	facets.add(new Facet(v1,v3,v2));
      	      }
      	      ii+=3;
      	    }
      	  }
      	}
      }
    }
    return facets;
  }

//--------------------------------------------------------------------//
/**
 * t@Zbg̒_Ԃ
 * @param edge Ӄ|C^
 * @param i <i>x</i> W|C^
 * @param j <i>y</i> W|C^
 * @param k <i>z</i> W|C^
 * @return ӂɑΉ钸_
 */
  protected Point3D facetVertex(int edge,int i,int j,int k){
    switch(edge){
      case 0:
      	return new Point3D(fx[i][j][k],y[j],z[k]);
      case 1:
      	return new Point3D(x[i+1],fy[i+1][j][k],z[k]);
      case 2:
      	return new Point3D(fx[i][j+1][k],y[j+1],z[k]);
      case 3:
      	return new Point3D(x[i],fy[i][j][k],z[k]);
      case 4:
      	return new Point3D(fx[i][j][k+1],y[j],z[k+1]);
      case 5:
      	return new Point3D(x[i+1],fy[i+1][j][k+1],z[k+1]);
      case 6:
      	return new Point3D(fx[i][j+1][k+1],y[j+1],z[k+1]);
      case 7:
      	return new Point3D(x[i],fy[i][j][k+1],z[k+1]);
      case 8:
      	return new Point3D(x[i],y[j],fz[i][j][k]);
      case 9:
      	return new Point3D(x[i+1],y[j],fz[i+1][j][k]);
      case 10:
      	return new Point3D(x[i+1],y[j+1],fz[i+1][j+1][k]);
      case 11:
      	return new Point3D(x[i],y[j+1],fz[i][j+1][k]);
    }
    return new Point3D();
  }
}
