package iso.component;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import gui.EditableList;
import gui.IntNumberSpinner;
import laf.WordMap;

//--------------------------------------------------------------------//
/**
 * V[PXt@CXgpl<br>
 * ̃f[^t@C摜쐬V[PXs
 */
public class SequenceFileListPanel extends JPanel{

/** t@Co͌` */
  protected static final String FORMAT[]={"PNG file","Windows bitmap"};

/** PNGt@C */
  public static final int PNG=0;

/** rbg}bvt@C */
  public static final int BMP=1;

/** t@CK\̏l */
  protected static final String INIT_REG_EX="flow\\d*.dat";

/** t@C̐lp^[ */
  protected static final Pattern
    NUM_PATTERN=Pattern.compile("\\D*(\\d*)\\D*");

/** JgfBNg */
  protected static File currentDirectory=new File(".");

/** \t[ */
  private ChannelFlowViewFrame frame;

/** t@CXg */
  protected EditableList list;

/** t@CK\̓tB[h */
  protected JTextField fileRegExField;

/** Xebvŏl̓Xsi[ */
  protected IntNumberSpinner stepMin;

/** Xebvől̓Xsi[ */
  protected IntNumberSpinner stepMax;

/** 摜o͌`IR{{bNX */
  protected JComboBox formatBox;

/**
 * V[PXt@CXgpl\z
 * @param frame \t[
 * @param list t@CXg
 */
  public SequenceFileListPanel(ChannelFlowViewFrame frame,
      	      	      	       EditableList list){
    super();
    this.frame=frame;
    this.list=list;
    setLayout(new BorderLayout());

    fileRegExField=new JTextField(INIT_REG_EX,20);
    fileRegExField.setBorder
      (new TitledBorder(WordMap.get(WordMap.REG_EX)));
    stepMin=new IntNumberSpinner
      (WordMap.get(WordMap.MIN_STEP),0,0,100000);
    stepMax=new IntNumberSpinner
      (WordMap.get(WordMap.MAX_STEP),10000,0,100000);
    formatBox=new JComboBox<String>(FORMAT);
    formatBox.setSelectedIndex(BMP);
    formatBox.setBorder
      (new TitledBorder(WordMap.get(WordMap.IMAGE_FORMAT)));

    JScrollPane listScr=new JScrollPane(list);
    listScr.setBorder(new TitledBorder(WordMap.get(WordMap.FILE_LIST)));
    add(listScr,BorderLayout.CENTER);
    add(getSetPanel(),BorderLayout.WEST);
    JPanel panel=new JPanel();
    panel.setLayout(new FlowLayout());
    panel.add(new JButton(new DeleteFileAction()));
    add(panel,BorderLayout.EAST);
  }

//--------------------------------------------------------------------//
/**
 * ݒplԂ
 * @return ݒpl
 */
  protected JPanel getSetPanel(){
    JPanel panel=new JPanel();
    panel.setLayout(new GridLayout(4,1));
    panel.add(fileRegExField);

    JPanel panel2=new JPanel();
    panel2.setLayout(new GridLayout(1,2));
    panel2.add(stepMin);
    panel2.add(stepMax);
    panel.add(panel2);

    JPanel panel3=new JPanel();
    panel3.setLayout(new FlowLayout());
    panel3.add(new JButton(new PickDirectoryAction()));
    panel.add(panel3);

    JPanel panel4=new JPanel();
    panel4.setLayout(new FlowLayout());
    panel4.add(formatBox);
    panel.add(panel4);
    return panel;
  }

//--------------------------------------------------------------------//
/**
 * p^[ɍt@Co
 * @param files t@Cz
 * @return p^[ɍt@C̃Xg
 */
  protected ArrayList<File> filtered(File files[]){
    ArrayList<File> fileList=new ArrayList<File>();
    Pattern pt=Pattern.compile(fileRegExField.getText());
    int sMin=stepMin.get();
    int sMax=stepMax.get();
    for(int i=0;i<files.length;i++){
      Matcher mt=pt.matcher(files[i].getName());
      if(mt.matches()){
      	Matcher mt2=NUM_PATTERN.matcher(files[i].getName());
      	mt2.find();
      	int step=Integer.parseInt(mt2.group(1));
      	if((sMin<=step) && (step<=sMax)){
      	  fileList.add(files[i]);
      	}
      }
    }
    fileList.trimToSize();
    Collections.sort(fileList,new FileComparator());
    return fileList;
  }

//--------------------------------------------------------------------//
/**
 * 摜o͌`Ԃ
 * @return 摜o͌`
 */
  public int getFormat(){
    return formatBox.getSelectedIndex();
  }

//--------------------------------------------------------------------//
/**
 * t@CofBNgIANV
 */
  public class PickDirectoryAction extends AbstractAction{

/**
 * ANV\z
 */
    public PickDirectoryAction(){
      putValue(NAME,WordMap.get(WordMap.PICK_DIRECTORY));
    }

//--------------------------------------------------------------------//
/**
 * t@CofBNgIt@CXg쐬
 * @param e ANVƂCxg
 */
    public void actionPerformed(ActionEvent e){
      JFileChooser fileChooser=new JFileChooser(currentDirectory);
      fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
      int ans=fileChooser.showOpenDialog(frame);
      if(ans!=fileChooser.APPROVE_OPTION) return;
      currentDirectory=fileChooser.getCurrentDirectory();
      File file=fileChooser.getSelectedFile();
      if(file.isDirectory()){
      	File files[]=file.listFiles();
      	ArrayList<File> fileList=filtered(files);
      	int count=fileList.size();
      	list.clear();
      	for(int i=0;i<count;i++){
      	  list.addElement(fileList.get(i));
      	}
      }
    }
  }

//--------------------------------------------------------------------//
/**
 * t@C폜ANV
 */
  public class DeleteFileAction extends AbstractAction{

/**
 * ANV\z
 */
    public DeleteFileAction(){
      putValue(NAME,WordMap.get(WordMap.REMOVE));
    }

//--------------------------------------------------------------------//
/**
 * t@C폜
 * @param e ANVƂCxg
 */
    public void actionPerformed(ActionEvent e){
      if(list.getSelectedIndex()>=0){
      	list.removeSelected();
      }
    }
  }

//--------------------------------------------------------------------//
/**
 * t@CrNX
 */
  public class FileComparator implements Comparator<File>{

/**
 * ANV\z
 */
    public FileComparator(){
    }

//--------------------------------------------------------------------//
/**
 * t@CotH_o
 * @param e ANVƂCxg
 */
    public int compare(File f1,File f2){
      Matcher mt1=NUM_PATTERN.matcher(f1.getName());
      Matcher mt2=NUM_PATTERN.matcher(f2.getName());
      if(mt1.find() && mt2.find()){
      	int n1=Integer.parseInt(mt1.group(1));
      	int n2=Integer.parseInt(mt2.group(1));
      	if(n1<n2)      return -1;
      	else if(n1>n2) return 1;
      	else           return 0;
      }
      else{
      	return 0;
      }
    }
  }
}
