package laf;
import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.NoSuchElementException;

//--------------------------------------------------------------------//
/**
 * `p[^i[}bv
 */
public class DrawParameterMap{

/** ^e[}\ */
  public static final String METAL_THEME="metaltheme";

/** f[^i[}bv */
  private Map<DrawParameterKey,Object> map;

/** f[^̏i[}bv */
  private Map<DrawParameterKey,Integer> orderMap;

/** f[^̏̃|C^ */
  private int order;

/**
 * `p[^i[}bv\z
 */
  public DrawParameterMap(){
    init();
  }

//--------------------------------------------------------------------//
/**
 * f[^
 */
  private void init(){
    map=new HashMap<DrawParameterKey,Object>();
    orderMap=new HashMap<DrawParameterKey,Integer>();
    order=0;
    add(DrawParameterKey.DECORATED,DrawParameter.OFF);
    add(DrawParameterKey.LAF_CLASS,DrawParameter.METAL_CLASS);
    VariableMetalTheme theme=new VariableMetalTheme();
    add(DrawParameterKey.THEME_NAME,theme.getName());
    for(int i=0;i<VariableMetalTheme.COLORS;i++){
      DrawParameterKey.THEME_COLORS[i]=
      	new DrawParameterKey(METAL_THEME,VariableMetalTheme.NAME[i]);
      add(DrawParameterKey.THEME_COLORS[i],theme.getColor(i));
    }
    add(DrawParameterKey.FONT_NAME,"Monospaced");
    add(DrawParameterKey.FONT_STYLE,new Integer(Font.PLAIN));
    add(DrawParameterKey.FONT_SIZE,new Integer(12));
    add(DrawParameterKey.BG_COLOR,Color.WHITE);
    add(DrawParameterKey.FG_COLOR,Color.BLACK);
    add(DrawParameterKey.SUB_COLOR,Color.PINK);
    add(DrawParameterKey.LANGUAGE,DrawParameter.ENGLISH);
  }

//--------------------------------------------------------------------//
/**
 * f[^}bvɒǉ
 * @param key }bṽL[
 * @param value i[l
 */
  protected void add(DrawParameterKey key,Object value){
    map.put(key,value);
    orderMap.put(key,new Integer(order));
    order++;
  }

//--------------------------------------------------------------------//
/**
 * T|[gĂL[o
 * @return T|[gĂL[i[z
 */
  public DrawParameterKey[] getSupportedKeys(){
    Iterator<DrawParameterKey> itr=map.keySet().iterator();
    ArrayList<DrawParameterKey> list=new ArrayList<DrawParameterKey>();
    while(itr.hasNext()){
      list.add(itr.next());
    }
    list.trimToSize();
    Collections.sort(list,new KeyComparator());
    int size=list.size();
    DrawParameterKey keys[]=new DrawParameterKey[size];
    for(int i=0;i<size;i++) keys[i]=(DrawParameterKey)list.get(i);
    return keys;
  }

//--------------------------------------------------------------------//
/**
 * f[^i[<br>
 * T|[gĂȂL[El̏ꍇ͉Ȃ
 * @param key }bṽL[
 * @param value i[l
 */
  public void put(DrawParameterKey key,Object value){
    Object oldValue=map.get(key);
    if(oldValue instanceof String){
      map.put(key,value);
    }
    else if(oldValue instanceof Number){
      if(value instanceof String){
      	try{
      	  map.put(key,Integer.valueOf((String)value));
      	}
      	catch(NumberFormatException ex){
      	}
      }
      else if(value instanceof Number){
      	map.put(key,value);
      }
    }
    else if(oldValue instanceof Color){
      if(value instanceof String){
      	try{
      	  map.put(key,new Color(Integer.parseInt((String)value,16)));
      	}
      	catch(NumberFormatException ex){
      	}
      }
      else if(value instanceof Color){
      	map.put(key,value);
      }
    }
  }

//--------------------------------------------------------------------//
/**
 * f[^i[<br>
 * T|[gĂȂL[El̏ꍇ͉Ȃ
 * @param key }bṽL[
 * @param value i[l
 */
  public void put(DrawParameterKey key,int value){
    put(key,new Integer(value));
  }

//--------------------------------------------------------------------//
/**
 * lɑΉ镶f[^o
 * @param key }bṽL[
 * @return L[ɑΉl̕\
 */
  public String getString(DrawParameterKey key){
    Object value=map.get(key);
    if(value instanceof String){
      return (String)value;
    }
    else if(value instanceof Number){
      return String.valueOf(value);
    }
    else if(value instanceof Color){
      int rgb=0x00FFFFFF&((Color)value).getRGB();
      return Integer.toString(rgb,16).toUpperCase();
    }
    else{
      return null;
    }
  }

//--------------------------------------------------------------------//
/**
 * tHgf[^Ԃ
 * @return L[ɑΉtHg
 */
  public Font getFont(){
    String name=(String)map.get(DrawParameterKey.FONT_NAME);
    Number style=(Number)map.get(DrawParameterKey.FONT_STYLE);
    Number size=(Number)map.get(DrawParameterKey.FONT_SIZE);
    return new Font(name,style.intValue(),size.intValue());
  }

//--------------------------------------------------------------------//
/**
 * Ff[^Ԃ<br>
 * ΉlFłȂꍇ null Ԃ
 * @param key }bṽL[
 * @return L[ɑΉF
 */
  public Color getColor(DrawParameterKey key){
    Object value=map.get(key);
    if(value instanceof Color) return (Color)value;
    else                       return null;
  }

//--------------------------------------------------------------------//
/**
 * L[̏̔rNX
 */
  private class KeyComparator implements Comparator<DrawParameterKey>{

/**
 * L[̏̔rIuWFNg\z
 */
    protected KeyComparator(){
    }

//--------------------------------------------------------------------//
/**
 * IuWFNg̔r
 * @param o1 rΏۂ̍ŏ̃IuWFNg
 * @param o2 rΏۂ 2 Ԗڂ̃IuWFNg
 * @return ŏ̏̕ 2 Ԗڂ̈菬ꍇ͕̐A<br>
 *         ꍇ 0A<br>
 *         ŏ̏̕ 2 Ԗڂ̈傫ꍇ͐̐
 */
    public int compare(DrawParameterKey o1,DrawParameterKey o2){
      int order1=((Number)orderMap.get(o1)).intValue();
      int order2=((Number)orderMap.get(o2)).intValue();
      if(order1<order2)      return -1;
      else if(order1>order2) return 1;
      else                   return 0;
    }
  }

//--------------------------------------------------------------------//
/**
 * 𕪊
 * @param s 镶
 * @return i[z
 */
  public static String[] split(String s){
    String ss[]=s.split("\\s+");
    if((ss.length==0) || !ss[0].equals("")){
      return ss;
    }
    else{
      String ss2[]=new String[ss.length-1];
      System.arraycopy(ss,1,ss2,0,ss.length-1);
      return ss2;
    }
  }
}
